#include "testlib.h"
#include <vector>

using namespace std;

void gen_rnd(int n, int h) {
    vector<int> a;
    for (int i = 0; i < n; i++) {
        a.push_back(rnd.next(0, h));
    }
    println(a.begin(), a.end());
}

void gen_large(int n, int h, int delta) {
    vector<int> a;
    for (int i = 0; i < n; i++) {
        a.push_back(rnd.next(h - delta, h));
    }
    println(a.begin(), a.end());
}

void gen_full(int n, int h) {
    gen_large(n, h, 0);
}

void gen_plateau(int n, int h) {
    vector<int> a;
    int delta = min(n + 29, h + 29) / 30;
    for (int i = 0; i < n; i++) {
        int t = rnd.next(n);
        if (t < n / 3) {
            a.push_back(rnd.next(1, h / 2));
        } else if (t < 2 * n / 3) {
            a.push_back(rnd.next(h / 2, h));
        } else {
            a.push_back(rnd.next(h / 2 - delta, h / 2 + delta));
        }
    }
    println(a.begin(), a.end());
}

void gen_2_plateaus(int n, int h) {
    vector<int> a;
    int delta = min(n + 29, h + 29) / 30;
    for (int i = 0; i < n; i++) {
        int t = rnd.next(n);
        if (t < n / 4) {
            a.push_back(rnd.next(0, delta));
        } else if (t < 2 * n / 4) {
            a.push_back(rnd.next(h - delta, h));
        } else {
            a.push_back(rnd.next(delta, h - delta));
        }
    }
    println(a.begin(), a.end());
}

void gen_square(int n, int h) {
    vector<int> a;
    int delta = h / n;
    if (delta > n / 2) {
        delta -= n / 2;
        int x = 1;
        for (int i = 0; i < n; i++) {
            a.push_back(x);
            x += delta;
            delta += 1;
            if (x > h) {
                x = h - 1;
            }
        }
    } else {
        delta = 1;
        int x = 1;
        for (int i = 0; i < n; i++) {
            a.push_back(x);
            x += delta;
            delta += 1;
            x %= h;
        }
    }
    println(a);
}

void gen_levels(int n, int h) {
    vector<int> a;
    int levels = max(10, min(n / 10, h / n));
    vector<int> t(levels);
    for (int i = 0; i < levels; i++) {
        t[i] = rnd.next(i * h / levels, (i + 1) * h / levels);
    }
    for (int i = 0; i < n; i++) {
        int x = t[rnd.next(levels)];
        a.push_back(max(0, min(h, rnd.next(x - h / levels / 20, x + h / levels / 20))));
    }
    println(a.begin(), a.end());
}

int main(int argc, char *argv[]) {

    registerGen(argc, argv, 1);

    int n = atoi(argv[1]), h = atoi(argv[2]), q = atoi(argv[3]);
    int gamma = atoi(argv[4]);

    println(5);
    println(n, h, q);
    println(gamma);

    gen_rnd(n, h);
    gen_large(n, h, h / 10);
    gen_plateau(n, h);
    gen_2_plateaus(n, h);
    gen_square(n, h);

}